package java.nio.file.spi;

import java.nio.file.attribute.BasicFileAttributes;
import java.nio.file.attribute.FileAttributeView;
import java.nio.file.LinkOption;
import java.nio.file.AccessMode;
import java.nio.file.FileStore;
import java.nio.file.CopyOption;
import java.nio.file.NoSuchFileException;
import java.nio.file.DirectoryStream;
import java.nio.channels.SeekableByteChannel;
import java.nio.channels.AsynchronousFileChannel;
import java.util.concurrent.ExecutorService;
import java.nio.channels.FileChannel;
import java.util.Set;
import java.nio.file.attribute.FileAttribute;
import java.util.HashSet;
import java.io.OutputStream;
import java.nio.channels.Channels;
import java.nio.file.Files;
import java.nio.file.StandardOpenOption;
import java.io.InputStream;
import java.nio.file.OpenOption;
import java.nio.file.Path;
import java.io.IOException;
import java.nio.file.FileSystem;
import java.util.Map;
import java.net.URI;
import java.util.Collections;
import java.security.AccessController;
import java.security.PrivilegedAction;
import java.nio.file.FileSystems;
import java.util.Iterator;
import java.util.ServiceLoader;
import java.util.ArrayList;
import java.util.List;

public abstract class FileSystemProvider
{
    private static final Object lock;
    private static volatile List<FileSystemProvider> installedProviders;
    private static boolean loadingProviders;
    
    private static Void checkPermission() {
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null) {
            securityManager.checkPermission(new RuntimePermission("fileSystemProvider"));
        }
        return null;
    }
    
    private FileSystemProvider(Void void1) {
    }
    
    protected FileSystemProvider() {
        this(checkPermission());
    }
    
    private static List<FileSystemProvider> loadInstalledProviders() {
        ArrayList<FileSystemProvider> list = new ArrayList();
        for (FileSystemProvider fileSystemProvider : ServiceLoader.load(FileSystemProvider.class, ClassLoader.getSystemClassLoader())) {
            String scheme = fileSystemProvider.getScheme();
            if (!scheme.equalsIgnoreCase("file")) {
                boolean n = false;
                Iterator<Object> iterator2 = list.iterator();
                while (iterator2.hasNext()) {
                    if (((FileSystemProvider)iterator2.next()).getScheme().equalsIgnoreCase(scheme)) {
                        n = true;
                        break;
                    }
                    continue;
                }
                if (n == 0) {
                    list.add(fileSystemProvider);
                }
                else {
                    continue;
                }
            }
            else {
                continue;
            }
        }
        return list;
    }
    
    public static List<FileSystemProvider> installedProviders() {
        if (FileSystemProvider.installedProviders == null) {
            FileSystemProvider provider = FileSystems.getDefault().provider();
            synchronized (FileSystemProvider.lock) {
                if (FileSystemProvider.installedProviders == null) {
                    if (FileSystemProvider.loadingProviders) {
                        throw new Error("Circular loading of installed providers detected");
                    }
                    FileSystemProvider.loadingProviders = true;
                    List list = (List)AccessController.doPrivileged(new PrivilegedAction<List<FileSystemProvider>>() {
                        @Override
                        public List<FileSystemProvider> run() {
                            return FileSystemProvider.access$000();
                        }
                    });
                    list.add(0, provider);
                    FileSystemProvider.installedProviders = Collections.unmodifiableList(list);
                }
            }
        }
        return FileSystemProvider.installedProviders;
    }
    
    public abstract String getScheme();
    
    public abstract FileSystem newFileSystem(URI p0, Map<String, ?> p1) throws IOException;
    
    public abstract FileSystem getFileSystem(URI p0);
    
    public abstract Path getPath(URI p0);
    
    public FileSystem newFileSystem(Path path, Map<String, ?> map) throws IOException {
        throw new UnsupportedOperationException();
    }
    
    public InputStream newInputStream(Path path, OpenOption... options) throws IOException {
        if (options.length > 0) {
            int length = options.length;
            int i = 0;
            while (i < length) {
                OpenOption obj = options[i];
                if (obj == StandardOpenOption.APPEND || obj == StandardOpenOption.WRITE) {
                    throw new UnsupportedOperationException("'" + obj + "' not allowed");
                }
                ++i;
            }
        }
        return Channels.newInputStream(Files.newByteChannel(path, options));
    }
    
    public OutputStream newOutputStream(Path path, OpenOption... array) throws IOException {
        int length = array.length;
        HashSet set = new HashSet(length + 3);
        if (length == 0) {
            set.add(StandardOpenOption.CREATE);
            set.add(StandardOpenOption.TRUNCATE_EXISTING);
        }
        else {
            int length2 = array.length;
            int i = 0;
            while (i < length2) {
                OpenOption openOption = array[i];
                if (openOption == StandardOpenOption.READ) {
                    throw new IllegalArgumentException("READ not allowed");
                }
                set.add(openOption);
                ++i;
            }
        }
        set.add(StandardOpenOption.WRITE);
        return Channels.newOutputStream(this.newByteChannel(path, set, new FileAttribute[0]));
    }
    
    public FileChannel newFileChannel(Path path, Set<? extends OpenOption> set, FileAttribute<?>... array) throws IOException {
        throw new UnsupportedOperationException();
    }
    
    public AsynchronousFileChannel newAsynchronousFileChannel(Path path, Set<? extends OpenOption> set, ExecutorService executorService, FileAttribute<?>... array) throws IOException {
        throw new UnsupportedOperationException();
    }
    
    public abstract SeekableByteChannel newByteChannel(Path p0, Set<? extends OpenOption> p1, FileAttribute<?>... p2) throws IOException;
    
    public abstract DirectoryStream<Path> newDirectoryStream(Path p0, DirectoryStream.Filter<? super Path> p1) throws IOException;
    
    public abstract void createDirectory(Path p0, FileAttribute<?>... p1) throws IOException;
    
    public void createSymbolicLink(Path path, Path path2, FileAttribute<?>... array) throws IOException {
        throw new UnsupportedOperationException();
    }
    
    public void createLink(Path path, Path path2) throws IOException {
        throw new UnsupportedOperationException();
    }
    
    public abstract void delete(Path p0) throws IOException;
    
    public boolean deleteIfExists(Path path) throws IOException {
        try {
            this.delete(path);
            return true;
        }
        catch (NoSuchFileException ex) {
            return false;
        }
    }
    
    public Path readSymbolicLink(Path path) throws IOException {
        throw new UnsupportedOperationException();
    }
    
    public abstract void copy(Path p0, Path p1, CopyOption... p2) throws IOException;
    
    public abstract void move(Path p0, Path p1, CopyOption... p2) throws IOException;
    
    public abstract boolean isSameFile(Path p0, Path p1) throws IOException;
    
    public abstract boolean isHidden(Path p0) throws IOException;
    
    public abstract FileStore getFileStore(Path p0) throws IOException;
    
    public abstract void checkAccess(Path p0, AccessMode... p1) throws IOException;
    
    public abstract <V extends FileAttributeView> V getFileAttributeView(Path p0, Class<V> p1, LinkOption... p2);
    
    public abstract <A extends BasicFileAttributes> A readAttributes(Path p0, Class<A> p1, LinkOption... p2) throws IOException;
    
    public abstract Map<String, Object> readAttributes(Path p0, String p1, LinkOption... p2) throws IOException;
    
    public abstract void setAttribute(Path p0, String p1, Object p2, LinkOption... p3) throws IOException;
    
    static List access$000() {
        return loadInstalledProviders();
    }
    
    static {
        lock = new Object();
        FileSystemProvider.loadingProviders = false;
    }
}
